# tf_cnn_benchmarks: High performance benchmarks with PowerAI DDL and Large Model Support

tf_cnn_benchmarks contains implementations of several popular convolutional
models, and is designed to be as fast as possible. tf_cnn_benchmarks supports
both running on a single machine or running in distributed mode across multiple
hosts. See the [High-Performance models
guide](https://www.tensorflow.org/performance/performance_models) for more
information.

These models utilize many of the strategies in the [TensorFlow Performance
Guide](https://www.tensorflow.org/performance/performance_guide). Benchmark
results can be found [here](https://www.tensorflow.org/performance/benchmarks).

These models are designed for performance. For models that have clean and
easy-to-read implementations, see the [TensorFlow Official
Models](https://github.com/tensorflow/models/tree/master/official).

These benchmarks are based on the TensorFlow 1.10 compatible branch of the TensorFlow benchmarks (branch: [cnn_tf_v1.10_compatible](https://github.com/tensorflow/benchmarks/tree/cnn_tf_v1.9_compatible/scripts/tf_cnn_benchmarks), commit: 1ef603fd7e568ff75127ec07f160808fcc59911c) and have been updated with the options for DDL and Large Model Support.

## Getting Started
Please make sure there is a conda environment activated with the `tf_cnn_benchmarks` package installed before going through the following steps.

To install the benchmarks run:

      $ install_tf_cnn_benchmarks <somedir>

To run ResNet50 with synthetic data without distortions with a single GPU, run

```
python $CONDA_PREFIX/tf_cnn_benchmarks/tf_cnn_benchmarks.py --num_gpus=1 --batch_size=32 --model=resnet50 --variable_update=parameter_server
```

Some important flags are

*   model: Model to use, e.g. resnet50, inception3, vgg16, and alexnet.
*   num_gpus: Number of GPUs to use.
*   data_dir: Path to data to process. If not set, synthetic data is used. To
    use Imagenet data use these
    [instructions](https://github.com/tensorflow/models/tree/master/research/inception#getting-started)
    as a starting point.
*   batch_size: Batch size for each GPU.
*   variable_update: The method for managing variables: parameter_server
    ,replicated, distributed_replicated, independent
*   local_parameter_device: Device to use as parameter server: cpu or gpu.

To see the full list of flags, run `python tf_cnn_benchmarks.py --help`.

## Distributed Version Using IBM PowerAI Distributed Deep Learning

The `tf_cnn_benchmarks.py` script has been modified to use IBM PowerAI Distributed Deep Learning (DDL). DDL can be used by passing `--variable_update=ddl` to `tf_cnn_benchmarks.py`.

Before using `--variable_update=ddl`, make sure there is a conda environment activated with the `tf_cnn_benchmarks` package installed:

The script should be launched using `ddlrun`. To run ResNet50 with synthetic data on two machines named host1 and host2:

```bash
$ ddlrun -H host1,host2 python tf_cnn_benchmarks.py --variable_update=ddl --model=resnet50  --num_gpus=1 --batch_size=128 --num_batches=1000000
```

For more information on DDL_OPTIONS and using ddlrun, see: `$CONDA_PREFIX/doc/ddl/README.md` and `$CONDA_PREFIX/doc/ddl-tensorflow/README.md`.

## Large Model Support

The `tf_cnn_benchmarks.py` script has been modified to use PowerAI TensorFlow Large Model Support.
The Large Model Support (LMS) allows larger batch sizes to be used when training the models.
The GoogleNet model can also be used with larger image sizes with LMS with the `image_size`
parameter.

To see the LMS flags and their descriptions, run `python tf_cnn_benchmarks.py --help`.

To achieve better scaling performance with LMS on multiple GPUs, the
 `tf_cnn_benchmarks.py` script should be run with the Distributed Deep
Learning `ddlrun` command.  Additionally, if running on a single system
without an Infiniband set up, the `--mpiarg -pami_noib` parameter must be added
to the ddlrun command line, for example:

```bash
ddlrun --mpiarg -pami_noib -H host1 python $CONDA_PREFIX/tf_cnn_benchmarks/tf_cnn_benchmarks.py --batch_size=512 --num_batches=100 --model=resnet50 --gpu_thread_mode=gpu_shared --num_gpus=1 --display_every=10 --lms=True  --variable_update=ddl
```

## Troubleshooting

### `--fp16_vars` Support with `--variable_update=ddl`

The `--fp16_vars` flag will not work with the `--variable_update=ddl` flag. For 16-bit floating point support with DDL, pass `--fp16_vars=False` and `--use_fp16=True` to `tf_cnn_benchmarks.py`. Then add `-pack 2.0` to the `--ddloption` flag of `ddlrun`. See the following example below:

```bash
$ ddlrun -H host1,host2 --ddloption "-pack 2.0" python $CONDA_PREFIX/tf_cnn_benchmarks/tf_cnn_benchmarks.py --variable_update=ddl --model=resnet50  --fp16_vars=False --use_fp16=True --num_gpus=1 --batch_size=128 --num_batches=1000000
```

### Using `ddlrun` without `--variable_update=ddl`

The `tf_cnn_benchmarks.py` does not support being launched from `ddlrun` without the `--variable_update=ddl` flag.

### Cleaning the Training Directory

The `tf_cnn_benchmarks.py` script writes to a training directory during its run. If the `--train_dir` flag is not passed, it will write to a unique temporary directory each time it is run. If the `--train_dir` flag is passed, it will write to the specified training directory. This directory will need to be emptied on each host before calling `tf_cnn_benchmarks.py` with a new model.

### Unsupported Models with DDL

HPM NASNet is currently not supported with DDL.
